#include <stdio.h>      
#include <stdlib.h>     
#include <unistd.h>
#include <getopt.h>  
#include <qapplication.h>
#include <qmainwindow.h>
#include <qmenubar.h>   
#include <qpopupmenu.h>
#include <qxml.h>
#include "parser.h"
#include "mixlaunch.h"
#include <alsa/asoundlib.h>

static struct option options[] =
        {{"help", 0, 0, 'h'},
         {"ctl_name", 1, 0, 'c'}, 
         {"gui_name", 1, 0, 'g'}, 
         {"mode", 1, 0, 'm'}, 
         {"verbose", 0, 0, 'v'},
         {"printDefaultGui", 0, 0, 'p'},
         {0, 0, 0, 0}};

int main(int argc, char *argv[])  
{
  QApplication *qApp = new QApplication(argc, argv);
  QMainWindow *top = new QMainWindow();
  top->setCaption("QAMix");
  int getopt_return;
  int option_index; 
  int channel = 0;
  int mode = 0;
  QString ctl_name = "hw:0";
  QString xml_name = "NO_NAME_SPECIFIED";
  bool verbose = false;
  bool printDefaultGui = false;
  int l1, deviceCount;
  QString qs, qs2, driver_name;
  snd_ctl_t *ctl_handle;
  snd_ctl_card_info_t *card_info;

  while((getopt_return = getopt_long(argc, argv, "hvpc:g:m:", options, &option_index)) >= 0) {
    switch(getopt_return) {
    case 'c':
        ctl_name = QString(optarg);
        break;
    case 'g':
        xml_name = QString(optarg);
        break;
    case 'm':
        mode = atoi(optarg);
        break;
    case 'h':
        printf("\n%s\n", aboutText.latin1());
        printf("-h, --help                      print this help message]\n");
        printf("-v, --verbose                   list all CTL elements\n");
        printf("-p, --printDefaultGui           print default GUI description\n");
        printf("-c <name>, --ctl_name <name>    name of CTL [hw:0]\n");
        printf("-g <name>, --gui_name <name>    GUI desciption file [deflt_mixer.xml]\n");
        printf("-m <num>,  --mode <num>         GUI mode (0=full, 1=master only, 2=simple\n");
        exit(EXIT_SUCCESS);
    case 'v':
        verbose = true;
        break;
    case 'p':
        printDefaultGui = true;
        break;
    }
  }
//  Mixer *mixer = new Mixer(ctl_name, xml_name, top);
  MixLaunch *mixLaunch = new MixLaunch(ctl_name, xml_name, mode, top);
//  if (verbose) {
//    mixer->listControls();
//  }
  if (printDefaultGui) {
    mixLaunch->printDefaultGui();
  }
  QPopupMenu *filePopup = new QPopupMenu(top); 
  QPopupMenu *viewPopup = new QPopupMenu(top); 
  viewPopup->setCheckable(true);
  QPopupMenu *devicePopup = new QPopupMenu(top); 
  devicePopup->setCheckable(true);
  QPopupMenu *aboutMenu = new QPopupMenu(top);
  mixLaunch->hide_ids[0] = top->menuBar()->insertItem("&File", filePopup);
  top->menuBar()->insertItem("&View", viewPopup);
  mixLaunch->hide_ids[1] = top->menuBar()->insertItem("&Soundcard", devicePopup);
  top->menuBar()->insertSeparator(); 
  mixLaunch->hide_ids[2] = top->menuBar()->insertItem("&About", aboutMenu); 
  filePopup->insertItem("&Open", mixLaunch, SLOT(openMixer()));
  filePopup->insertItem("&Quit", qApp, SLOT(quit()));
  viewPopup->insertItem("&Full", 0, 0);
  viewPopup->insertItem("&Master only", 1, 1); 
  viewPopup->insertItem("&Simple", 2, 2);
  aboutMenu->insertItem("About QAMix", mixLaunch, SLOT(displayAbout())); 

  deviceCount = 0;
  for (l1 = 0; l1 < 8; l1++) {
    qs.sprintf("hw:%d", l1);
    if (snd_ctl_open(&ctl_handle, qs.latin1(), SND_CTL_NONBLOCK) < 0) {
//      fprintf(stderr, "Could not open CTL %s.\n", qs.latin1());
    } else { 
      snd_ctl_card_info_alloca(&card_info);
      snd_ctl_card_info(ctl_handle, card_info);
      driver_name = QString(snd_ctl_card_info_get_driver(card_info));
      qs2.sprintf("%s (%s)", driver_name.latin1(), qs.latin1());
      devicePopup->insertItem(qs2, deviceCount, deviceCount);
      if (qs == ctl_name.stripWhiteSpace()) {
        devicePopup->setItemChecked(deviceCount, true);
      }
      deviceCount++;
      snd_ctl_close(ctl_handle);
    }
  }
  viewPopup->setItemChecked(0, true);
  QObject::connect(devicePopup, SIGNAL(activated(int)), mixLaunch, SLOT(newDevice(int)));
  QObject::connect(viewPopup, SIGNAL(activated(int)), mixLaunch, SLOT(newMode(int)));
  QObject::connect(qApp, SIGNAL(aboutToQuit()), mixLaunch, SLOT(closeMixer()));
  top->setCentralWidget(mixLaunch); 
  top->show();
  qApp->setMainWidget(top); 
  return qApp->exec(); 
}
