#include <stdio.h>
#include <stdlib.h>
#include <qstring.h>
#include <qwidget.h>
#include <qfiledialog.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include "mixlaunch.h"
#include "mixer.h"

MixLaunch::MixLaunch(QString p_ctlname, QString p_xml_name, int p_mode, QWidget *parent, const char *name) : QVBox(parent, name) {

  mainWindow = (QMainWindow *)parent;
  int l1;

  ctl_name = p_ctlname;
  xml_name = p_xml_name;
  mode = p_mode;
  if (mode == 1) {
    mainWindow->menuBar()->hide();
  }
  aboutWidget = new QMessageBox(this);
  mixer = new Mixer(ctl_name, xml_name, mode, this);
  updateCaption();
}

MixLaunch::~MixLaunch() {

  if (mixer) {
    delete(mixer);
  }  
}

void MixLaunch::openMixer() {
 
  if (!(xml_name = QString(QFileDialog::getOpenFileName("/usr/share/qamix", "QAMix GUI description (*.xml)")))) {
    return;
  } else {
    closeMixer();
//    printf("Opening new mixer %s\n", xml_name.latin1());
    mixer = new Mixer(ctl_name, xml_name, mode, this);
    mixer->show();
  }
}

void MixLaunch::closeMixer() {

//  printf("Closing mixer\n");
  if (mixer) {
    delete(mixer);
  }
}

void MixLaunch::displayAbout() {

    aboutWidget->about(this, "About QAMix", aboutText);
    aboutWidget->raise();
}

void MixLaunch::newDevice(int id) {

  int l1, index;
  QString qs;

  QPopupMenu *deviceMenu = (QPopupMenu *)sender();
  
  for (l1 = 0; l1 < deviceMenu->count(); l1++) {
    deviceMenu->setItemChecked(l1, false);
  }
  deviceMenu->setItemChecked(id, true);
  qs = deviceMenu->text(id);
//  printf("deviceMenu->text(%d): %s\n", id, qs.latin1()); 
  index = qs.findRev("(hw:");
  ctl_name = qs.mid(index + 1, 4);
  xml_name = "NO_NAME_SPECIFIED"; // Automatic search for GUI description
//  printf("ctl_name: %s\n", ctl_name.latin1());
  closeMixer();
  mixer = new Mixer(ctl_name, xml_name, mode, this);
  mixer->show();
  updateCaption();
}

void MixLaunch::updateCaption() {

  QString driver_name, qs;
  snd_ctl_card_info_t *card_info;

  snd_ctl_card_info_alloca(&card_info);
  snd_ctl_card_info(mixer->hctlData->ctl_handle, card_info);
  driver_name = QString(snd_ctl_card_info_get_driver(card_info));
  qs.sprintf("QAMix 0.0.6  -  %s", driver_name.latin1());
  mainWindow->setCaption(qs);
}

void MixLaunch::printDefaultGui() {

  mixer->printDefaultGui();
}

void MixLaunch::newMode(int id) {

  int l1;

  QPopupMenu *viewMenu = (QPopupMenu *)sender();
  mode = id;
  if (mode == 1) {
    for (l1 = 0; l1 < 3; l1++) {
      mainWindow->menuBar()->setItemVisible(hide_ids[l1], false);
    }
  } else {
   for (l1 = 0; l1 < 3; l1++) {
      mainWindow->menuBar()->setItemVisible(hide_ids[l1], true);
    }
  }
  for (l1 = 0; l1 < viewMenu->count(); l1++) {
    viewMenu->setItemChecked(l1, false);
  }
  viewMenu->setItemChecked(mode, true);
  closeMixer();
  mixer = new Mixer(ctl_name, xml_name, mode, this);
  mixer->show();  
}
